/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <pwd.h>
#include <grp.h>

#include "common.h"
#include "options.h"
#include "log.h"
#include "thread.h"
#include "conf.h"
#include "crypt.h"
#include "pdetach.h"
#include "server.h"
#include "caller.h"
#include "packet.h"
#include "main.h"


int main(int argc, char *argv[]) {
	extern options_t options;
	extern conf_t conf;
	server_t *server;

	log_init(NULL);	/* init logging routines */
	rand_init();	/* init random data generator */


	/* Options */ {
		options_init();

		unless (options_parse(argc,argv)) {
			ERROR("%s: failed to parse options",*argv);
				return EXIT_FAILURE;
		}
	
		if (options.help) {
			usage();
			return EXIT_SUCCESS;
		} else if (options.license) {
			license();
			return EXIT_SUCCESS;
		} else if (options.version) {
			version();
			return EXIT_SUCCESS;
		}
	} /* end options */

	/* Configuration */ {
		conf_init();

		unless (conf_parse()) {
			ERROR("Failed to parse configuration.");
				return EXIT_FAILURE;
		}
	} /* end Configuration */

	log_init(&conf);

	INFO("AnonNet started.");
	
	unless (change_privileges()) {
		ERROR("There was a problem changing privileges.");
		INFO("AnonNet exiting.");
		return EXIT_FAILURE;
	}
		
	if (conf.daemon) {
		DEBUG("Forking process.");
		if (!pdetach()) {
			ERROR("Error forking.");
			INFO("AnonNet exiting.");
			return EXIT_FAILURE;
		}
	}


	if ((server = server_create()) == NULL) {
		ERROR("Could not get a node server.");
		INFO("AnonNet exiting.");
		return EXIT_FAILURE;
	}


	thread_run(server);


	if (thread_is_alive(server) == false) {
		ERROR("Could not start node server.");
		INFO("AnonNet exiting.");
		return EXIT_FAILURE;
	}


	if (options.debug)
		debug();


	for (;;) {
		sleep(1);
		if (thread_is_alive(server) == false) {
			WARN("Node server is not alive.");
			INFO("AnonNet exiting.");
			return EXIT_FAILURE;
		}
	}

	INFO("AnonNet exiting.");
	return EXIT_SUCCESS;
}


/*
 * Need to set umask (600?)! probably elsewhere so it gets changed
 * regardless if we're chaning uid/gid. also should drop env variables.
 * alter ulimit and (scheduling?) priority. login class?
 */
bool change_privileges() {
	extern conf_t conf;
	struct group *group;
	struct passwd *passwd;
		

	if (conf.group != NULL) {
		if ((group = getgrnam(conf.group)) != NULL || (group = getgrgid((gid_t)atoi(conf.group))) != NULL) {
			if (setgid(group->gr_gid) != 0) {
				WARN("could not change to group %s: %s",group->gr_name,strerror(errno));
				return false;
			} else {
				INFO("changed to group %s",group->gr_name);
			}
		} else {
			WARN("could not change to group %s: invalid group or ENOMEM",conf.group);
			return false;
		}
	}
	
	if (conf.user != NULL) {
		if ((passwd = getpwnam(conf.user)) != NULL || (passwd = getpwuid((uid_t)atoi(conf.user))) != NULL) {
			if (setuid(passwd->pw_uid) != 0) {
				WARN("could not change to user %s: %s",passwd->pw_name,strerror(errno));
				return false;
			} else {
				INFO("changed to user %s",passwd->pw_name);
			}
		} else {
			WARN("could not change to user %s: invalid user or ENOMEM",conf.user);
			return false;
		}
	}
	
	return true;
}


void debug() {
	unsigned char request[1 + 256 + 2];
	caller_t *caller;
	int sv[2];

	socketpair(AF_LOCAL,SOCK_STREAM,0,sv);

	FLAG_SET(request[0],0x04); /* NAME */
	strcpy(&request[1],"127.0.0.1");
	*((uint16_t *)&request[257]) = htons(8080);

	if ((caller = caller_create(sv[1])) == NULL) {
		ERROR("Could not get caller for debugging.");
		return;
	}

	thread_run(caller);

	write(sv[0],request,sizeof(request));


	return; /* void */
}



void usage() {
	printf("%s\n",USAGE);
}

void license() {
	printf("%s\n",LICENSE);
}

void version() {
	printf("%s %s\n",PACKAGE,VERSION);
}
